;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x2290dafe5cb0434f8654ed72f5e54bc3 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xfd84d303526182d319a2da7b64c06268 . "./code.cg2js/cg-printing.fasl")
(#x1fbf8aba46637d269de945692c96caab . "./code.cg2js/cg-caret.fasl")
(#x1f1612cf8779478c79fb6731aa1fd2ed . "./code.cg2js/cg-item-list.fasl")
(#x0f67f75d715bed65950673cbd6c2dd8f . "./code.cg2js/cg-progress-indicator.fasl")
(#xb70cf44a3c42e726b3b4cadc58a446e6 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x1ccffc84f4351badbaed0f1acf61995d . "./code.cg2js/cg-outline.fasl")
(#x0f9f04b6a2ca0229b115a78732bcb9d6 . "./code.cg2js/cg-palette.fasl")
(#xafb44036865b0111cbf445660898a0de . "./code.cg2js/cg-selected-object.fasl")
(#xaeac01a58dc7a07a942c5c14b5673583 . "./code.cg2js/cg-status-bar.fasl")
(#xe1fe7e74f6e5c92dec3019225e40ebb8 . "./code.cg2js/cg-wrap-string.fasl")
(#xb174821da5fa9f332c13ff7055b74c79 . "./code.cg2js/cg-fill-texture.fasl")
(#xd9c4dd44f706f77406940a1737559360 . "./code.cg2js/cg-layout.fasl")
(#xb238b40ab020a2efb01e4b6c2e4a06c1 . "./code.cg2js/cg-timer.fasl")
(#xc6bdfdac26b3f0db59de644f84fd44c7 . "./code.cg2js/cg-dialog-item.fasl")
(#xce83d9bd3e941a48cad0aebdc381410d . "./code.cg2js/cg-button.fasl")
(#x88b1fb345325a9f0e5d508bfb623e7f9 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x4201b7d969dc8d3364df530d4ef758b1 . "./code.cg2js/cg-tray.fasl")
(#xe7b886f41d9fb1930ecf781057742361 . "./code.cg2js/cg-plot-widget.fasl")
(#x036fa68b7ed04126f245d438f8e0a3f5 . "./code.cg2js/cg-pixmap.fasl")
(#x4734dd553176998d802381993410d025 . "./code.cg2js/cg-font-dialog.fasl")
(#xca9d6c6eda01db9188eac3d002560efb . "./code.cg2js/cg-os-widget.fasl")
(#xb38011b42f87b20f12dd290574fc8c6b . "./code.cg2js/cg-text-or-combo.fasl")
(#x645651193d6d885b9ae7f9264d1b2718 . "./code.cg2js/cg-trackbar.fasl")
(#xa5f1b3402139c52e3348f0cabe8ffa2b . "./code.cg2js/cg-up-down-control.fasl")
(#xc93db1641d0cf4dc072263d61b57d502 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xb7c55af06df14ea8c7731211b046899a . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x06566d3737796d25050dc53e954c545a . "./code.cg2js/cg-intersect-posbox.fasl")
(#xed8053af9d9f42f62508066eb9bef5b3 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x345ba5c746a2085b6b658efd95987962 . "./code.cg2js/cg-chart-widget.fasl")
(#x81767a80a6d931e417ebb912c26e4a7d . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x75c1ecd1f83494b27d06fea83e9ebc0d . "./code.cg2js/cg-group-box.fasl")
(#x544eb7d9b618ca85fedf59ec42eccea8 . "./code.cg2js/cg-static-text.fasl")
(#x90c24dd5bff0d4eab40bf98b6cc063a1 . "./code.cg2js/cg-class-slot-grid.fasl")
(#xebac168970725fc80d47a7376e3a532d . "./code.cg2js/cg-color-dialog.fasl")
(#x233d1de8ff0d3bc06214816ec470a80a . "./code.cg2js/cg-intersect.fasl")
(#xe40f61aaa3c16997dc72c43eb7781f9f . "./code.cg2js/cg-gesture-emulation.fasl")
(#x793261616a668a4bf363fcd89b979f99 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x2613af83b447f34f814d766ea6246ec7 . "./code.cg2js/cg-html-dialog.fasl")
(#x95729340d81837dbf092a8a416e74fd3 . "./code.cg2js/cg-lettered-menu.fasl")
(#x646038e2e086b33d5b6166aa65575b39 . "./code.cg2js/cg-class-grid.fasl")
(#x3f23b62d2d083daa6686c02e1b0be687 . "./code.cg2js/cg-pixmap-printing.fasl")
(#xd609fce0ace3e68048ebf873c1c70fd6 . "./code.cg2js/cg-calendar.fasl")
(#x99c50329be5969951abc91a1c6af6bb0 . "./code.cg2js/cg-editable-text.fasl")
(#x2c4bb7ab16ad94c7696061087c4c858b . "./code.cg2js/cg-property.fasl")
(#xe40094f056e10ddb854f178320aa8c08 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xc78edd43683bb7e6b11f1db2e4970ff3 . "./code.cg2js/cg-grid-widget.fasl")
(#x32a987254b3bc477451c5b8322b4d575 . "./code.cg2js/cg-paren-matching.fasl")
(#xba7640880e3b9de17b405cf044b69e3b . "./code.cg2js/cg-dev.fasl")
(#x84548536b2e53e6c1f64f7382ee5a756 . "./code.cg2js/cg-drawable.fasl")
(#x33998edb2c6623126dd0a28ea738511c . "./code.cg2js/cg-curve.fasl")
(#x7360cfe54d1a284b463eff245ec19ee9 . "./code.cg2js/cg-lisp-widget.fasl")
(#xea565ff134efa7a7e876a34affde665e . "./code.cg2js/cg-dde-utils.fasl")
(#xb7b1b4bf21ac2278051dabba55c7832b . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x8c24dfc8109f42749b9accc9a12ac789 . "./code.cg2js/cg-object-editor-layout.fasl")
(#xf5caf96c631da86b34c27876ab099906 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xa985d2e3778d1e4350d9b9b949357046 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xe26dc7258939619f25a2b771067064e7 . "./code.cg2js/cg-lamp.fasl")
(#x02903ff1351ad31d0cdcab15046dfb78 . "./code.cg2js/cg-common-control.fasl")
(#xa5ea1c6ef3d8f255ff6d3f9d14ef8301 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x0a983935aad4d8d1b802cdd1db7f5b36 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x3599ad87df2cc22908b79bc8e7c50c4e . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x310f1a6e3762699f9812e739ce4f652d . "./code.cg2js/cg-project-window.fasl")
(#x57dba6a06a38798fb96ec2e0baafe244 . "./code.cg2js/cg-sample-file-menu.fasl")
(#xa5377345b8991d3baf7bff7d6f1f2806 . "./code.cg2js/cg-class-support.fasl")
(#xbc54889677cfba07cd13d8557fd132f4 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x7030aec44b330ba74503dd504fe50e93 . "./code.cg2js/cg-yes-no-list.fasl")
(#x17e3038e113c97496d942caee181c293 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x41cebfc059b1a7c89d17079c74738599 . "./code.cg2js/cg-string-dialog.fasl")
(#x63d55716a2d3aae923a2646f9803cf1a . "./code.cg2js/cg-menu-tooltip.fasl")
(#x412bcfda1df3f9e9d1d4f4bcfd914890 . "./code.cg2js/cg-dropping-outline.fasl")
(#x4ece7a44dbaaa13868909e4ea10845c1 . "./code.cg2js/cg.fasl")
(#x688a30b5cfcda6de8708a5bea5931e75 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x5f643c570129493f0d4547acf3672f26 . "./code.cg2js/cg-scaling-stream.fasl")
(#x91eabe23dc655aba34fdd53db5733c66 . "./code.cg2js/cg-yes-no-string.fasl")
(#x57ec6ce29a4429a6812f45eef1a756fe . "./code.cg2js/cg-icon-pixmap.fasl")
(#xa80ab7e1cd57fc9146b8ae1bf4769ed0 . "./code.cg2js/cg-web-browser.fasl")
(#xe13a30b9f51c7c21de0fce328dd6772b . "./code.cg2js/cg-clipboard-stack.fasl")
(#x75f1759de4c4a0bd8925f4af8e8bd825 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x7c6e12b5dd399be608a6a7832543b4b8 . "./code.cg2js/cg-message-dialog.fasl")
(#x7bbc4b8ac1728a1676728d0f1dea86e8 . "./code.cg2js/cg-comtab.fasl")
(#xda332b2cc36befa8f88634f5a1c7a677 . "./code.cg2js/cg-base.fasl")
(#x3acd650e4c9d272971d68724e0501b8a . "./code.cg2js/cg-pixmap-widget.fasl")
(#x3fafb067224b0a2fd960c75dee0d9f06 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#xd437abaaec5e8ace7732dda271042cfd . "./code.cg2js/cg-text-edit-pane.fasl")
(#xade80c9cf81fbac03657aa50c0cba673 . "./code.cg2js/cg-utility-dialog.fasl")
(#x6bb198a271c3d52a36fbb1e175d85eff . "./code.cg2js/cg-object-editor.fasl")
(#xfe8fd74a61b178c1b34c5213295d2533 . "./code.cg2js/cg-edit-in-place.fasl")
(#xcbc7fc15d820054c60360de3e744a552 . "./code.cg2js/cg-radio-button.fasl")
(#xda0f7a9b27663cf92a7f6e480207c371 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xfc26e05e28e77826b88f8080c2c0fa96 . "./code.cg2js/cg-nodes.fasl")
(#xf9fd78735c7e892bab157a1c8b7ccf04 . "./code.cg2js/cg-get-position.fasl")
(#xef6781590606f8b0639f8338c1618bb1 . "./code.cg2js/cg-directory-dialog.fasl")
(#xcde7cb883f6513714b59095cf8acf2a9 . "./code.cg2js/cg-lisp-text.fasl")
(#x7fe3ab7a352afd59c55091c4898d6f5c . "./code.cg2js/cg-combo-box.fasl")
(#xfd5d027e375769ef0b7b1419bddfd0d8 . "./code.cg2js/cg-text-widget.fasl")
(#xd6d0dfaf43d41bb9ae6b6e5b9d4931a9 . "./code.cg2js/cg-list-view.fasl")
(#x06302f491281ddd786e9ef78cc3a531b . "./code.cg2js/cg-header-control.fasl")
(#x323f60693a71218a9e8fe112255b7f7b . "./code.cg2js/cg-graphics-context.fasl")
(#x2484df579132026e13a2f36d97aca938 . "./code.cg2js/cg-get-pixmap.fasl")
(#x743829b0e8b72a7104d8c38eecb8d91c . "./code.cg2js/cg-menu.fasl")
(#x34860b78bffc65bc7b7eaa849932149c . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x677d67178fe4f8af6df3575eb9a7b556 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x8537f4a3612129d2784f315958b8ecf3 . "./code.cg2js/cg-tooltip.fasl")
(#xfc583ae138c474715282f09b7211d247 . "./code.cg2js/cg-html-widget.fasl")
(#x735f6dce0be5573d0d4da3db82774d3c . "./code.cg2js/cg-choice-list.fasl")
(#xf049cc3fc5e2f10109b99e61af7cd006 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x769ac9b48a37cb150b6162915bbea187 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x7c9b6cf42cf3c5664cc512094b4b4aa9 . "./code.cg2js/cg-icon.fasl")
(#x88e22feb982a163d177086def20184b3 . "./code.cg2js/cg-acache.fasl")
(#x49b5b12674ba4b6cb1b5699b1fae6696 . "./code.cg2js/cg-template-string.fasl")
(#x1a1f9558ec0b77b530f20c4c44c274c7 . "./code.cg2js/cg-bitmap-pane.fasl")
(#xf3b222181a6b523e73ea5c05f013af7a . "./code.cg2js/cg-toggling-widget.fasl")
(#x07fc44d42f07454762bb8c26fe18fd4a . "./code.cg2js/cg-tab-control.fasl")
(#x66d442c4e4b2d2ebd5ea2c4efe177455 . "./code.cg2js/cg-split-bar.fasl")
(#x431e600fc31cfd4b8632f09a8f36ff1c . "./code.cg2js/cg-picture-widget.fasl")
(#x168a7344f5b52908b8eb18ae1bfd9755 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xeaba00f38eae1bd026ae105706af3d2c . "./code.cg2js/cg-choose-printer.fasl")
(#x5b1ec77ae6c5cf5380b9ea9b1c1f6a58 . "./code.cg2js/cg-os-window.fasl")
(#x20f4c3b279cc92a6351d023ee0015a5e . "./code.cg2js/cg-scroll-bar.fasl")
(#x00e93158895ea18db62f7f731adfd473 . "./code.cg2js/cg-clipboard.fasl")
(#x5b6f3600905f816cfb0440bdc18925b0 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x75026224e93251a4bfb4f67c4ab6ad81 . "./code.cg2js/cg-hotspot.fasl")
(#x794e13c4704dfedb7b10538683ab0401 . "./code.cg2js/cg-rich-edit.fasl")
(#xde96fe3fe065f189ae317ad9d022253c . "./code.cg2js/cg-check-box.fasl")
(#x197029174d3808345e07da913ecd3d7d . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x072bd135b7213d32602d86474947b101 . "./code.cg2js/cg-file-dialog.fasl")
(#x8f3dce8056c8f09ec21b5877152c7d0c . "./code.cg2js/cg-toolbar.fasl")
(#x0b99b6019d05657b5145aabcb6148ea5 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xba04711df61cfc709cc3d7212479e6c7 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x213afa263d0a5e67f349cf02139a281f . "./code.cg2js/cg-shortcut-menu.fasl")
(#x317845c97234d3fbbd89da054bf54b29 . "./code.idejs/ide-project.fasl")
(#x8ad892a4276e8c47233acc7e1f7ac040 . "./code.idejs/ide-intro.fasl")
(#x96327104656091577ea9ecdc53e63e5b . "./code.idejs/ide-find-in-files.fasl")
(#x68f2f2625b86dc55756196d96988d6f2 . "./code.idejs/ide-grapher.fasl")
(#x301ea94745925fa477b407cc91ebfcac . "./code.idejs/ide-inspector.fasl")
(#xa0808a6daac7793765c3e68966de61dd . "./code.idejs/ide-editor.fasl")
(#xe14cd4c971d7947191dc2cd8f5f60d91 . "./code.idejs/ide-window-list.fasl")
(#xffc9cfcdad51cc5f1ba308d22de0aa4a . "./code.idejs/ide-process-browser.fasl")
(#x0e33657462fcdec464c2423f5bda6b56 . "./code.idejs/ide-patch-dialog.fasl")
(#x3cebce83148e39def9baf41bff635e13 . "./code.idejs/ide-class-grapher.fasl")
(#xc0bfbf7894c39698c922f7814ea22a4c . "./code.idejs/ide-base.fasl")
(#x70804fa1616b1b94744d73d45c3d725b . "./code.idejs/ide.fasl")
(#x486818cfe0beccca1afaeb62f45e7641 . "./code.idejs/ide-trace.fasl")
(#x7282ee44813d05dc8d9de2801548b5a4 . "./code.idejs/ide-form.fasl")
(#x1181c9c62df4195f81e4bf510ba8eddd . "./code.idejs/ide-find-definitions.fasl")
(#x17b5851df4ae96400b157aed0d806f50 . "./code.idejs/ide-apropos.fasl")
(#xe82b3c1873f855670eb493aad1991497 . "./code.idejs/ide-class-browser.fasl")
(#xd33a80c96fb15c402092bb9df99cedeb . "./code.idejs/ide-navigator.fasl")
(#xb1889e516f203e2cf27d291571a75d05 . "./code.idejs/ide-clipboard.fasl")
(#x986ad3cb7adfad5c2c9a4a3b3d89c9ac . "./code.idejs/ide-acache.fasl")
(#x0162cc04e44b7b4b62c4d299fb16c2ac . "./code.idejs/ide-keyshort.fasl")
(#x171f914311d8bf44f144e08227b5e004 . "./code.idejs/ide-package-list.fasl")
(#xd2bed3401373a7b5402c80683991d753 . "./code.idejs/ide-dev.fasl")
(#xb19a66ec109be5a69fb5293c577fc104 . "./cgjs.html")
(#x34779e5d4bbb71d45bf63f88fac54100 . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js/" "code.idejs/"))
    (format s "
(let ((path (make-pathname
	     :type \"fasl\"
	     :directory (pathname-directory
			 (merge-pathnames
			  ~s
			  (translate-logical-pathname \"sys:\"))))))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
